/*
 *  ebmodule.c - EB library wapper module for Python
 *               Tamito KAJIYAMA <2 February 2001>
 *
 *  $Id: ebmodule.c,v 1.2 2001/09/18 09:18:34 kajiyama Exp $
 */

#include "Python.h"

#include <eb/eb.h>
#include <eb/appendix.h>
#include <eb/text.h>
#include <eb/error.h>
#include <eb/font.h>
#include <eb/language.h>

/* global variables */
static PyObject *EBError;
static PyObject *CallbackContext;

/*********************************************************************
 *  Book object
 */

typedef struct {
  PyObject_HEAD
  EB_Book book;
} BookObject;

static struct PyMethodDef Book_methods[] = {
  {NULL, NULL}  /* sentinel */
};

staticforward PyTypeObject BookType;

static BookObject *
book_new(void)
{
  BookObject *self;
	
  self = PyObject_NEW(BookObject, &BookType);
  if (self == NULL)
    return NULL;
  eb_initialize(&self->book);
  return self;
}

static void
book_dealloc(BookObject *self)
{
  PyMem_DEL(self);
}

static PyObject *
book_getattr(BookObject *self, char *name)
{
  return Py_FindMethod(Book_methods, (PyObject *)self, name);
}

static char BookType__doc__[] =
"A data structure for representing Book objects.";

static PyTypeObject BookType = {
  PyObject_HEAD_INIT(&PyType_Type)
  0,				/*ob_size*/
  "EB_Book",			/*tp_name*/
  sizeof(BookObject),		/*tp_basicsize*/
  0,				/*tp_itemsize*/
  /* methods */
  (destructor)book_dealloc,	/*tp_dealloc*/
  (printfunc)0,			/*tp_print*/
  (getattrfunc)book_getattr,	/*tp_getattr*/
  (setattrfunc)0,		/*tp_setattr*/
  (cmpfunc)0,			/*tp_compare*/
  (reprfunc)0,			/*tp_repr*/
  0,				/*tp_as_number*/
  0,				/*tp_as_sequence*/
  0,				/*tp_as_mapping*/
  (hashfunc)0,			/*tp_hash*/
  (ternaryfunc)0,		/*tp_call*/
  (reprfunc)0,			/*tp_str*/
  
  /* Space for future expansion */
  0L,0L,0L,0L,
  BookType__doc__ /* Documentation string */
};

/*********************************************************************
 *  Appendix object
 */

typedef struct {
  PyObject_HEAD
  EB_Appendix appendix;
} AppendixObject;

static struct PyMethodDef Appendix_methods[] = {
  {NULL, NULL}  /* sentinel */
};

staticforward PyTypeObject AppendixType;

static AppendixObject *
appendix_new(void)
{
  AppendixObject *self;
	
  self = PyObject_NEW(AppendixObject, &AppendixType);
  if (self == NULL)
    return NULL;
  eb_initialize_appendix(&self->appendix);
  return self;
}

static void
appendix_dealloc(AppendixObject *self)
{
  PyMem_DEL(self);
}

static PyObject *
appendix_getattr(AppendixObject *self, char *name)
{
  return Py_FindMethod(Appendix_methods, (PyObject *)self, name);
}

static char AppendixType__doc__[] =
"A data structure for representing Appendix objects.";

static PyTypeObject AppendixType = {
  PyObject_HEAD_INIT(&PyType_Type)
  0,				/*ob_size*/
  "EB_Appendix",		/*tp_name*/
  sizeof(AppendixObject),	/*tp_basicsize*/
  0,				/*tp_itemsize*/
  /* methods */
  (destructor)appendix_dealloc,	/*tp_dealloc*/
  (printfunc)0,			/*tp_print*/
  (getattrfunc)appendix_getattr,/*tp_getattr*/
  (setattrfunc)0,		/*tp_setattr*/
  (cmpfunc)0,			/*tp_compare*/
  (reprfunc)0,			/*tp_repr*/
  0,				/*tp_as_number*/
  0,				/*tp_as_sequence*/
  0,				/*tp_as_mapping*/
  (hashfunc)0,			/*tp_hash*/
  (ternaryfunc)0,		/*tp_call*/
  (reprfunc)0,			/*tp_str*/
  
  /* Space for future expansion */
  0L,0L,0L,0L,
  AppendixType__doc__ /* Documentation string */
};

/*********************************************************************
 *  Hookset object
 */

typedef struct {
  PyObject_HEAD
  EB_Hookset hookset;
  PyObject *callbacks;
} HooksetObject;

static struct PyMethodDef Hookset_methods[] = {
  {NULL, NULL}  /* sentinel */
};

staticforward PyTypeObject HooksetType;

static HooksetObject *
hookset_new(void)
{
  HooksetObject *self;
	
  self = PyObject_NEW(HooksetObject, &HooksetType);
  if (self == NULL)
    return NULL;
  eb_initialize_hookset(&self->hookset);
  self->callbacks = PyDict_New();
  return self;
}

static void
hookset_dealloc(HooksetObject *self)
{
  Py_DECREF(self->callbacks);
  PyMem_DEL(self);
}

static PyObject *
hookset_getattr(HooksetObject *self, char *name)
{
  return Py_FindMethod(Hookset_methods, (PyObject *)self, name);
}

static char HooksetType__doc__[] =
"A data type for representing Hookset objects.";

static PyTypeObject HooksetType = {
  PyObject_HEAD_INIT(&PyType_Type)
  0,				/*ob_size*/
  "EB_Hookset",			/*tp_name*/
  sizeof(HooksetObject),	/*tp_basicsize*/
  0,				/*tp_itemsize*/
  /* methods */
  (destructor)hookset_dealloc,	/*tp_dealloc*/
  (printfunc)0,			/*tp_print*/
  (getattrfunc)hookset_getattr,	/*tp_getattr*/
  (setattrfunc)0,		/*tp_setattr*/
  (cmpfunc)0,			/*tp_compare*/
  (reprfunc)0,			/*tp_repr*/
  0,				/*tp_as_number*/
  0,				/*tp_as_sequence*/
  0,				/*tp_as_mapping*/
  (hashfunc)0,			/*tp_hash*/
  (ternaryfunc)0,		/*tp_call*/
  (reprfunc)0,			/*tp_str*/
  
  /* Space for future expansion */
  0L,0L,0L,0L,
  HooksetType__doc__ /* Documentation string */
};

/*********************************************************************
 *  StringBuffer object
 */

typedef struct {
  PyObject_HEAD
  PyObject *buffer;
} BufferObject;

static char buffer_get__doc__[] = "\
Buffer.get() => string

Return the string that the object retains.";

static PyObject *
buffer_get(BufferObject *self, PyObject *args)
{
  if (!PyArg_ParseTuple(args, ""))
    return NULL;
  Py_INCREF(self->buffer);
  return self->buffer;
}

static char buffer_set__doc__[] = "\
Buffer.set(string) => None

Store the given string into the object.";

static PyObject *
buffer_set(BufferObject *self, PyObject *args)
{
  char *string;

  if (!PyArg_ParseTuple(args, "s", &string))
    return NULL;
  Py_DECREF(self->buffer);
  self->buffer = PyString_FromString(string);
  Py_INCREF(Py_None);
  return Py_None;
}

static struct PyMethodDef Buffer_methods[] = {
  {"get", (PyCFunction)buffer_get, METH_VARARGS, buffer_get__doc__},
  {"set", (PyCFunction)buffer_set, METH_VARARGS, buffer_set__doc__},
  {NULL, NULL}  /* sentinel */
};

staticforward PyTypeObject BufferType;

static BufferObject *
buffer_new(void)
{
  BufferObject *self;
	
  self = PyObject_NEW(BufferObject, &BufferType);
  if (self == NULL)
    return NULL;
  self->buffer = Py_None;
  Py_INCREF(Py_None);
  return self;
}

static void
buffer_dealloc(BufferObject *self)
{
  Py_DECREF(self->buffer);
  PyMem_DEL(self);
}

static PyObject *
buffer_getattr(BufferObject *self, char *name)
{
  return Py_FindMethod(Buffer_methods, (PyObject *)self, name);
}

static char BufferType__doc__[] =
"A data structure for retaining a string.";

static PyTypeObject BufferType = {
  PyObject_HEAD_INIT(&PyType_Type)
  0,				/*ob_size*/
  "Buffer",			/*tp_name*/
  sizeof(BufferObject),		/*tp_basicsize*/
  0,				/*tp_itemsize*/
  /* methods */
  (destructor)buffer_dealloc,	/*tp_dealloc*/
  (printfunc)0,			/*tp_print*/
  (getattrfunc)buffer_getattr,	/*tp_getattr*/
  (setattrfunc)0,		/*tp_setattr*/
  (cmpfunc)0,			/*tp_compare*/
  (reprfunc)0,			/*tp_repr*/
  0,				/*tp_as_number*/
  0,				/*tp_as_sequence*/
  0,				/*tp_as_mapping*/
  (hashfunc)0,			/*tp_hash*/
  (ternaryfunc)0,		/*tp_call*/
  (reprfunc)0,			/*tp_str*/
  
  /* Space for future expansion */
  0L,0L,0L,0L,
  BufferType__doc__ /* Documentation string */
};

static char mod_Buffer__doc__[] = "\
Buffer() => buffer

Return a new instance of the EB_Buffer object.";

static PyObject *
mod_Buffer(PyObject *self, PyObject *args)
{
  if (!PyArg_ParseTuple(args, ""))
    return NULL;
  return (PyObject *)buffer_new();
}

/*********************************************************************
 *  Module functions
 */

/* [eb.h] */

/* copyright.c */

static char py_eb_have_copyright__doc__[] = "\
eb_have_copyright(book) => int

Return true if the current subbook has a copyright notice.";

static PyObject *
py_eb_have_copyright(PyObject *self, PyObject *args)
{
  BookObject *book;

  if (!PyArg_ParseTuple(args, "O!", &BookType, &book))
    return NULL;
  return PyInt_FromLong(eb_have_copyright(&book->book));
}

static char py_eb_copyright__doc__[] = "\
eb_copyright(book, position) => (int, int)

Return a position of the copyright notice.  Return None if failed.";

static PyObject *
py_eb_copyright(PyObject *self, PyObject *args)
{
  BookObject *book;
  EB_Position pos;

  if (!PyArg_ParseTuple(args, "O!", &BookType, &book))
    return NULL;
  if (eb_copyright(&book->book, &pos) < 0) {
    Py_INCREF(Py_None);
    return Py_None;
  }
  return Py_BuildValue("(ii)", pos.page, pos.offset);
}

/* eb.c */

static char py_eb_Book__doc__[] = "\
EB_Book() => book

Return a new instance of the Book object.";

static PyObject *
py_eb_Book(PyObject *self, PyObject *args)
{
  if (!PyArg_ParseTuple(args, ""))
    return NULL;
  return (PyObject *)book_new();
}

static char py_eb_bind__doc__[] = "\
eb_bind(book, string) => int

Bind the book object to an electronic book in the given path.  Return -1 if failed.";

static PyObject *
py_eb_bind(PyObject *self, PyObject *args)
{
  BookObject *book;
  char *path;

  if (!PyArg_ParseTuple(args, "O!s", &BookType, &book, &path))
    return NULL;
  return PyInt_FromLong(eb_bind(&book->book, path));
}

static char py_eb_suspend__doc__[] = "\
eb_suspend(book) => None

Suspend the use of the book.  The current subbook, language, and font are to be unselected.";

static PyObject *
py_eb_suspend(PyObject *self, PyObject *args)
{
  BookObject *book;

  if (!PyArg_ParseTuple(args, "O!", &BookType, &book))
    return NULL;
  eb_suspend(&book->book);
  Py_INCREF(Py_None);
  return Py_None;
}

static char py_eb_clear__doc__[] = "\
eb_clear(book) => None

End up the use of the book.  The book will be unbound.";

static PyObject *
py_eb_clear(PyObject *self, PyObject *args)
{
  BookObject *book;

  if (!PyArg_ParseTuple(args, "O!", &BookType, &book))
    return NULL;
  eb_clear(&book->book);
  Py_INCREF(Py_None);
  return Py_None;
}

static char py_eb_is_bound__doc__[] = "\
eb_is_bound(book) => int

Return true if the book object is bound to an electronic book.";

static PyObject *
py_eb_is_bound(PyObject *self, PyObject *args)
{
  BookObject *book;

  if (!PyArg_ParseTuple(args, "O!", &BookType, &book))
    return NULL;
  return PyInt_FromLong(eb_is_bound(&book->book));
}

static char py_eb_path__doc__[] = "\
eb_path(book) => string

Return the path of the electronic book.  Return None if failed.";

static PyObject *
py_eb_path(PyObject *self, PyObject *args)
{
  BookObject *book;
  const char *path;

  if (!PyArg_ParseTuple(args, "O!", &BookType, &book))
    return NULL;
  if ((path = eb_path(&book->book)) == NULL) {
    Py_INCREF(Py_None);
    return Py_None;
  }
  return PyString_FromString(path);
}

static char py_eb_disc_type__doc__[] = "\
eb_disc_type(book) => int

Return the disc type of the book.  Return -1 if failed.";

static PyObject *
py_eb_disc_type(PyObject *self, PyObject *args)
{
  BookObject *book;

  if (!PyArg_ParseTuple(args, "O!", &BookType, &book))
    return NULL;
  return PyInt_FromLong(eb_disc_type(&book->book));
}

static char py_eb_character_code__doc__[] = "\
eb_character_code(book) => int

Return the character code of the book.  Return -1 if failed.";

static PyObject *
py_eb_character_code(PyObject *self, PyObject *args)
{
  BookObject *book;

  if (!PyArg_ParseTuple(args, "O!", &BookType, &book))
    return NULL;
  return PyInt_FromLong(eb_character_code(&book->book));
}

/* filename.c */

/* graphic.c */

static char py_eb_have_graphic_search__doc__[] = "\
eb_have_graphic_search(book) => int

Return true if the book has the graphic search facility.";

static PyObject *
py_eb_have_graphic_search(PyObject *self, PyObject *args)
{
  BookObject *book;

  if (!PyArg_ParseTuple(args, "O!", &BookType, &book))
    return NULL;
  return PyInt_FromLong(eb_have_graphic_search(&book->book));
}

/* keyword.c */

static char py_eb_have_keyword_search__doc__[] = "\
eb_have_keyword_search(book) => int

Return true if the book has the keyword search facility.";

static PyObject *
py_eb_have_keyword_search(PyObject *self, PyObject *args)
{
  BookObject *book;

  if (!PyArg_ParseTuple(args, "O!", &BookType, &book))
    return NULL;
  return PyInt_FromLong(eb_have_keyword_search(&book->book));
}

/* menu.c */

static char py_eb_have_menu__doc__[] = "\
eb_have_menu(book) => int

Return true if the book has a menu.";

static PyObject *
py_eb_have_menu(PyObject *self, PyObject *args)
{
  BookObject *book;

  if (!PyArg_ParseTuple(args, "O!", &BookType, &book))
    return NULL;
  return PyInt_FromLong(eb_have_menu(&book->book));
}

static char py_eb_menu__doc__[] = "\
eb_menu(book) => (int, int)

Return the position of the book's menu.  Return None if failed.";

static PyObject *
py_eb_menu(PyObject *self, PyObject *args)
{
  BookObject *book;
  EB_Position pos;

  if (!PyArg_ParseTuple(args, "O!", &BookType, &book))
    return NULL;
  if (eb_menu(&book->book, &pos) < 0) {
    Py_INCREF(Py_None);
    return Py_None;
  }
  return Py_BuildValue("(ii)", pos.page, pos.offset);
}

/* multi.c */

static char py_eb_have_multi_search__doc__[] = "\
eb_have_multi_search(book) => int

Return true if the book has the multiple search facility.";

static PyObject *
py_eb_have_multi_search(PyObject *self, PyObject *args)
{
  BookObject *book;

  if (!PyArg_ParseTuple(args, "O!", &BookType, &book))
    return NULL;
  return PyInt_FromLong(eb_have_multi_search(&book->book));
}

/* search.c */

static char py_eb_have_exactword_search__doc__[] = "\
eb_have_exactword_search(book) => int

Return true if the book has the \"exact word\" search facility.";

static PyObject *
py_eb_have_exactword_search(PyObject *self, PyObject *args)
{
  BookObject *book;

  if (!PyArg_ParseTuple(args, "O!", &BookType, &book))
    return NULL;
  return PyInt_FromLong(eb_have_exactword_search(&book->book));
}

static char py_eb_search_exactword__doc__[] = "\
eb_search_exactword(book, string) => int

Issue an \"exact word\" search.  Return -1 if failed.";

static PyObject *
py_eb_search_exactword(PyObject *self, PyObject *args)
{
  BookObject *book;
  const char *query;

  if (!PyArg_ParseTuple(args, "O!s", &BookType, &book, &query))
    return NULL;
  return PyInt_FromLong(eb_search_exactword(&book->book, query));
}

static char py_eb_have_word_search__doc__[] = "\
eb_have_word_search(book) => int

Return true if the book has the word search facility.";

static PyObject *
py_eb_have_word_search(PyObject *self, PyObject *args)
{
  BookObject *book;

  if (!PyArg_ParseTuple(args, "O!", &BookType, &book))
    return NULL;
  return PyInt_FromLong(eb_have_word_search(&book->book));
}

static char py_eb_search_word__doc__[] = "\
eb_search_word(book, string) => int

Issue a word search.  Return -1 if failed.";

static PyObject *
py_eb_search_word(PyObject *self, PyObject *args)
{
  BookObject *book;
  const char *query;

  if (!PyArg_ParseTuple(args, "O!s", &BookType, &book, &query))
    return NULL;
  return PyInt_FromLong(eb_search_word(&book->book, query));
}

static char py_eb_have_endword_search__doc__[] = "\
eb_have_endword_search(book) => int

Return true if the book has the \"end word\" search facility.";

static PyObject *
py_eb_have_endword_search(PyObject *self, PyObject *args)
{
  BookObject *book;

  if (!PyArg_ParseTuple(args, "O!", &BookType, &book))
    return NULL;
  return PyInt_FromLong(eb_have_endword_search(&book->book));
}

static char py_eb_search_endword__doc__[] = "\
eb_search_endword(book, string) => int

Issue an \"end word\" search.  Return -1 if failed.";

static PyObject *
py_eb_search_endword(PyObject *self, PyObject *args)
{
  BookObject *book;
  const char *query;

  if (!PyArg_ParseTuple(args, "O!s", &BookType, &book, &query))
    return NULL;
  return PyInt_FromLong(eb_search_endword(&book->book, query));
}

#define MAX_HITS 50

static char py_eb_hit_list__doc__[] = "\
eb_hit_list(book) => ((int, int) (int, int))[]

Return a tuple of hits obtained by the last search.  Each hit is represented by a pair of positions for the heading and body of a section.  The number of hits returned by a call is limited to 50, so that an application program have to call the function until an empty tuple is returned in order to get all hits.  Return None if failed.";

static PyObject *
py_eb_hit_list(PyObject *self, PyObject *args)
{
  BookObject *book;
  EB_Hit hits[MAX_HITS];
  PyObject *tuple;
  int len, i;

  if (!PyArg_ParseTuple(args, "O!", &BookType, &book))
    return NULL;
  if ((len = eb_hit_list(&book->book, hits, MAX_HITS)) < 0) {
    Py_INCREF(Py_None);
    return Py_None;
  }
  if ((tuple = PyTuple_New(len)) == NULL)
    return NULL;
  for (i = 0; i < len; i++)
    PyTuple_SetItem(tuple, i, Py_BuildValue("((ii)(ii))",
					    hits[i].heading.page,
					    hits[i].heading.offset,
					    hits[i].text.page,
					    hits[i].text.offset));
  return tuple;
}

/* subbook.c */

static char py_eb_initialize_all_subbooks__doc__[] = "\
eb_initialize_all_subbooks(book) => int

Initialize all subbook in the book.  Return -1 if failed.";

static PyObject *
py_eb_initialize_all_subbooks(PyObject *self, PyObject *args)
{
  BookObject *book;

  if (!PyArg_ParseTuple(args, "O!", &BookType, &book))
    return NULL;
  return PyInt_FromLong(eb_initialize_all_subbooks(&book->book));
}

static char py_eb_subbook_count__doc__[] = "\
eb_subbook_count(book) => int

Return the number of subbooks in the book.  Return -1 if failed.";

static PyObject *
py_eb_subbook_count(PyObject *self, PyObject *args)
{
  BookObject *book;

  if (!PyArg_ParseTuple(args, "O!", &BookType, &book))
    return NULL;
  return PyInt_FromLong(eb_subbook_count(&book->book));
}

static char py_eb_subbook_list__doc__[] = "\
eb_subbook_list(book) => int[]

Return a tuple of subbook codes in the book.  Return None if failed.";

static PyObject *
py_eb_subbook_list(PyObject *self, PyObject *args)
{
  BookObject *book;
  EB_Subbook_Code sublist[EB_MAX_SUBBOOKS];
  PyObject *tuple;
  int len, i;

  if (!PyArg_ParseTuple(args, "O!", &BookType, &book))
    return NULL;
  if ((len = eb_subbook_list(&book->book, sublist)) < 0) {
    Py_INCREF(Py_None);
    return Py_None;
  }
  if ((tuple = PyTuple_New(len)) == NULL)
    return NULL;
  for (i = 0; i < len; i++)
    PyTuple_SetItem(tuple, i, PyInt_FromLong(sublist[i]));
  return tuple;
}

static char py_eb_subbook__doc__[] = "\
eb_subbook(book) => int

Return the current subbook's code.  Return -1 if failed.";

static PyObject *
py_eb_subbook(PyObject *self, PyObject *args)
{
  BookObject *book;

  if (!PyArg_ParseTuple(args, "O!", &BookType, &book))
    return NULL;
  return PyInt_FromLong(eb_subbook(&book->book));
}

static char py_eb_subbook_title__doc__[] = "\
eb_subbook_title(book) => string

Return the title of the current subbook.  Return None if failed.";

static PyObject *
py_eb_subbook_title(PyObject *self, PyObject *args)
{
  BookObject *book;
  const char *title;

  if (!PyArg_ParseTuple(args, "O!", &BookType, &book))
    return NULL;
  if ((title = eb_subbook_title(&book->book)) == NULL) {
    Py_INCREF(Py_None);
    return Py_None;
  }
  return PyString_FromString(title);
}

static char py_eb_subbook_title2__doc__[] = "\
eb_subbook_title2(book, int) => string

Return the title of the subbook specified with the given subbook code.  Return None if failed.";

static PyObject *
py_eb_subbook_title2(PyObject *self, PyObject *args)
{
  BookObject *book;
  EB_Subbook_Code subbook;
  const char *title;

  if (!PyArg_ParseTuple(args, "O!i", &BookType, &book, &subbook))
    return NULL;
  if ((title = eb_subbook_title2(&book->book, subbook)) == NULL) {
    Py_INCREF(Py_None);
    return Py_None;
  }
  return PyString_FromString(title);
}

static char py_eb_subbook_directory__doc__[] = "\
eb_subbook_directory(book) => string

Return the directory of the current subbook.  Return None if failed.";

static PyObject *
py_eb_subbook_directory(PyObject *self, PyObject *args)
{
  BookObject *book;
  const char *directory;

  if (!PyArg_ParseTuple(args, "O!", &BookType, &book))
    return NULL;
  if ((directory = eb_subbook_directory(&book->book)) == NULL) {
    Py_INCREF(Py_None);
    return Py_None;
  }
  return PyString_FromString(directory);
}

static char py_eb_subbook_directory2__doc__[] = "\
eb_subbook_directory2(book, int) => string

Return the directory of the subbook specified with the given subbook code.  Return None if failed.";

static PyObject *
py_eb_subbook_directory2(PyObject *self, PyObject *args)
{
  BookObject *book;
  EB_Subbook_Code subbook;
  const char *directory;

  if (!PyArg_ParseTuple(args, "O!i", &BookType, &book, &subbook))
    return NULL;
  if ((directory = eb_subbook_directory2(&book->book, subbook)) == NULL) {
    Py_INCREF(Py_None);
    return Py_None;
  }
  return PyString_FromString(directory);
}

static char py_eb_set_subbook__doc__[] = "\
eb_set_subbook(book, int) => int

Specify the current subbook.  Return -1 if failed.";

static PyObject *
py_eb_set_subbook(PyObject *self, PyObject *args)
{
  BookObject *book;
  EB_Subbook_Code subbook;

  if (!PyArg_ParseTuple(args, "O!i", &BookType, &book, &subbook))
    return NULL;
  return PyInt_FromLong(eb_set_subbook(&book->book, subbook));
}

static char py_eb_unset_subbook__doc__[] = "\
eb_unset_subbook(book) => None

Unselect the current subbook.";

static PyObject *
py_eb_unset_subbook(PyObject *self, PyObject *args)
{
  BookObject *book;

  if (!PyArg_ParseTuple(args, "O!", &BookType, &book))
    return NULL;
  eb_unset_subbook(&book->book);
  Py_INCREF(Py_None);
  return Py_None;
}

/* [appendix.h] */

/* appendix.c */

static char py_eb_Appendix__doc__[] = "\
EB_Appendix() => appendix

Return a new instance of the Appendix object.";

static PyObject *
py_eb_Appendix(PyObject *self, PyObject *args)
{
  if (!PyArg_ParseTuple(args, ""))
    return NULL;
  return (PyObject *)appendix_new();
}

static char py_eb_clear_appendix__doc__[] = "";
static PyObject *
py_eb_clear_appendix(PyObject *self, PyObject *args)
{
  AppendixObject *appendix;

  if (!PyArg_ParseTuple(args, "O!", &AppendixType, &appendix))
    return NULL;
  eb_clear_appendix(&appendix->appendix);
  Py_INCREF(Py_None);
  return Py_None;
}

static char py_eb_suspend_appendix__doc__[] = "";
static PyObject *
py_eb_suspend_appendix(PyObject *self, PyObject *args)
{
  AppendixObject *appendix;

  if (!PyArg_ParseTuple(args, "O!", &AppendixType, &appendix))
    return NULL;
  eb_suspend_appendix(&appendix->appendix);
  Py_INCREF(Py_None);
  return Py_None;
}

static char py_eb_bind_appendix__doc__[] = "";
static PyObject *
py_eb_bind_appendix(PyObject *self, PyObject *args)
{
  AppendixObject *appendix;
  char *path;

  if (!PyArg_ParseTuple(args, "O!s", &AppendixType, &appendix, &path))
    return NULL;
  return PyInt_FromLong(eb_bind_appendix(&appendix->appendix, path));
}

static char py_eb_is_appendix_bound__doc__[] = "";
static PyObject *
py_eb_is_appendix_bound(PyObject *self, PyObject *args)
{
  AppendixObject *appendix;

  if (!PyArg_ParseTuple(args, "O!", &AppendixType, &appendix))
    return NULL;
  return PyInt_FromLong(eb_is_appendix_bound(&appendix->appendix));
}

static char py_eb_appendix_path__doc__[] = "";
static PyObject *
py_eb_appendix_path(PyObject *self, PyObject *args)
{
  AppendixObject *appendix;
  const char *path;

  if (!PyArg_ParseTuple(args, "O!", &AppendixType, &appendix))
    return NULL;
  if (!eb_is_appendix_bound(&appendix->appendix) ||
      (path = eb_appendix_path(&appendix->appendix)) == NULL) {
    Py_INCREF(Py_None);
    return Py_None;
  }
  return PyString_FromString(path);
}

/* appsub.c */

static char py_eb_initialize_appendix_subbook__doc__[] = "";
static PyObject *
py_eb_initialize_appendix_subbook(PyObject *self, PyObject *args)
{
  AppendixObject *appendix;

  if (!PyArg_ParseTuple(args, "O!", &AppendixType, &appendix))
    return NULL;
  return PyInt_FromLong(eb_initialize_appendix_subbook(&appendix->appendix));
}

static char py_eb_initialize_all_appendix_subbooks__doc__[] = "";
static PyObject *
py_eb_initialize_all_appendix_subbooks(PyObject *self, PyObject *args)
{
  AppendixObject *appendix;

  if (!PyArg_ParseTuple(args, "O!", &AppendixType, &appendix))
    return NULL;
  return PyInt_FromLong(eb_initialize_all_appendix_subbooks(&appendix->appendix));
}

static char py_eb_appendix_subbook_count__doc__[] = "";
static PyObject *
py_eb_appendix_subbook_count(PyObject *self, PyObject *args)
{
  AppendixObject *appendix;

  if (!PyArg_ParseTuple(args, "O!", &AppendixType, &appendix))
    return NULL;
  return PyInt_FromLong(eb_appendix_subbook_count(&appendix->appendix));
}

static char py_eb_appendix_subbook_list__doc__[] = "";
static PyObject *
py_eb_appendix_subbook_list(PyObject *self, PyObject *args)
{
  AppendixObject *appendix;
  EB_Subbook_Code sublist[EB_MAX_SUBBOOKS];
  PyObject *tuple;
  int len, i;

  if (!PyArg_ParseTuple(args, "O!", &AppendixType, &appendix))
    return NULL;
  if ((len = eb_appendix_subbook_list(&appendix->appendix, sublist)) < 0)
    return PyInt_FromLong(-1);
  if ((tuple = PyTuple_New(len)) == NULL)
    return NULL;
  for (i = 0; i < len; i++)
    PyTuple_SetItem(tuple, i, PyInt_FromLong(sublist[i]));
  return tuple;
}

static char py_eb_appendix_subbook__doc__[] = "";
static PyObject *
py_eb_appendix_subbook(PyObject *self, PyObject *args)
{
  AppendixObject *appendix;

  if (!PyArg_ParseTuple(args, "O!", &AppendixType, &appendix))
    return NULL;
  return PyInt_FromLong(eb_appendix_subbook(&appendix->appendix));
}

static char py_eb_appendix_subbook2__doc__[] = "";
static PyObject *
py_eb_appendix_subbook2(PyObject *self, PyObject *args)
{
  AppendixObject *appendix;
  char *name;

  if (!PyArg_ParseTuple(args, "O!s", &AppendixType, &appendix, &name))
    return NULL;
  return PyInt_FromLong(eb_appendix_subbook2(&appendix->appendix, name));
}

static char py_eb_appendix_subbook_directory__doc__[] = "";
static PyObject *
py_eb_appendix_subbook_directory(PyObject *self, PyObject *args)
{
  AppendixObject *appendix;
  const char *dirname;

  if (!PyArg_ParseTuple(args, "O!", &AppendixType, &appendix))
    return NULL;
  dirname = eb_appendix_subbook_directory(&appendix->appendix);
  if (dirname == NULL) {
    Py_INCREF(Py_None);
    return Py_None;
  }
  return PyString_FromString(dirname);
}

static char py_eb_appendix_subbook_directory2__doc__[] = "";
static PyObject *
py_eb_appendix_subbook_directory2(PyObject *self, PyObject *args)
{
  AppendixObject *appendix;
  EB_Subbook_Code code;
  const char *dirname;

  if (!PyArg_ParseTuple(args, "O!i", &AppendixType, &appendix, &code))
    return NULL;
  dirname = eb_appendix_subbook_directory2(&appendix->appendix, code);
  if (dirname == NULL) {
    Py_INCREF(Py_None);
    return Py_None;
  }
  return PyString_FromString(dirname);
}

static char py_eb_set_appendix_subbook__doc__[] = "";
static PyObject *
py_eb_set_appendix_subbook(PyObject *self, PyObject *args)
{
  AppendixObject *appendix;
  EB_Subbook_Code subbook;

  if (!PyArg_ParseTuple(args, "O!i", &AppendixType, &appendix, &subbook))
    return NULL;
  return PyInt_FromLong(eb_set_appendix_subbook(&appendix->appendix, subbook));
}

static char py_eb_unset_appendix_subbook__doc__[] = "";
static PyObject *
py_eb_unset_appendix_subbook(PyObject *self, PyObject *args)
{
  AppendixObject *appendix;

  if (!PyArg_ParseTuple(args, "O!", &AppendixType, &appendix))
    return NULL;
  eb_unset_appendix_subbook(&appendix->appendix);
  Py_INCREF(Py_None);
  return Py_None;
}

/* stopcode.c */

static char py_eb_have_stopcode__doc__[] = "";
static PyObject *
py_eb_have_stopcode(PyObject *self, PyObject *args)
{
  AppendixObject *appendix;

  if (!PyArg_ParseTuple(args, "O!", &AppendixType, &appendix))
    return NULL;
  return PyInt_FromLong(eb_have_stopcode(&appendix->appendix));
}

static char py_eb_stopcode__doc__[] = "";
static PyObject *
py_eb_stopcode(PyObject *self, PyObject *args)
{
  AppendixObject *appendix;

  if (!PyArg_ParseTuple(args, "O!", &AppendixType, &appendix))
    return NULL;
  return PyInt_FromLong(eb_stopcode(&appendix->appendix));
}

/* narwalt.c */

static char py_eb_have_narrow_alt__doc__[] = "";
static PyObject *
py_eb_have_narrow_alt(PyObject *self, PyObject *args)
{
  AppendixObject *appendix;

  if (!PyArg_ParseTuple(args, "O!", &AppendixType, &appendix))
    return NULL;
  return PyInt_FromLong(eb_have_narrow_alt(&appendix->appendix));
}

static char py_eb_narrow_alt_start__doc__[] = "";
static PyObject *
py_eb_narrow_alt_start(PyObject *self, PyObject *args)
{
  AppendixObject *appendix;

  if (!PyArg_ParseTuple(args, "O!", &AppendixType, &appendix))
    return NULL;
  return PyInt_FromLong(eb_narrow_alt_start(&appendix->appendix));
}

static char py_eb_narrow_alt_end__doc__[] = "";
static PyObject *
py_eb_narrow_alt_end(PyObject *self, PyObject *args)
{
  AppendixObject *appendix;

  if (!PyArg_ParseTuple(args, "O!", &AppendixType, &appendix))
    return NULL;
  return PyInt_FromLong(eb_narrow_alt_end(&appendix->appendix));
}

static char py_eb_narrow_alt_character_text__doc__[] = "";
static PyObject *
py_eb_narrow_alt_character_text(PyObject *self, PyObject *args)
{
  AppendixObject *appendix;
  BufferObject *buffer;
  int ch, ret;
  char workbuf[EB_MAXLEN_TEXT_WORK + 1];

  if (!PyArg_ParseTuple(args, "O!iO!",
                        &AppendixType, &appendix, &ch, &BufferType, &buffer))
    return NULL;
  ret = eb_narrow_alt_character_text(&appendix->appendix, ch, workbuf);
  if (ret >= 0) {
    Py_DECREF(buffer->buffer);
    buffer->buffer = PyString_FromString(workbuf);
  }
  return PyInt_FromLong(ret);
}

static char py_eb_forward_narrow_alt_character__doc__[] = "";
static PyObject *
py_eb_forward_narrow_alt_character(PyObject *self, PyObject *args)
{
  AppendixObject *appendix;
  int ch, n;

  if (!PyArg_ParseTuple(args, "O!ii", &AppendixType, &appendix, &ch, &n))
    return NULL;
  return PyInt_FromLong
    (eb_forward_narrow_alt_character(&appendix->appendix, ch, n));
}

static char py_eb_backward_narrow_alt_character__doc__[] = "";
static PyObject *
py_eb_backward_narrow_alt_character(PyObject *self, PyObject *args)
{
  AppendixObject *appendix;
  int ch, n;

  if (!PyArg_ParseTuple(args, "O!ii", &AppendixType, &appendix, &ch, &n))
    return NULL;
  return PyInt_FromLong
    (eb_backward_narrow_alt_character(&appendix->appendix, ch, n));
}

/* widealt.c */

static char py_eb_have_wide_alt__doc__[] = "";
static PyObject *
py_eb_have_wide_alt(PyObject *self, PyObject *args)
{
  AppendixObject *appendix;

  if (!PyArg_ParseTuple(args, "O!", &AppendixType, &appendix))
    return NULL;
  return PyInt_FromLong(eb_have_wide_alt(&appendix->appendix));
}

static char py_eb_wide_alt_start__doc__[] = "";
static PyObject *
py_eb_wide_alt_start(PyObject *self, PyObject *args)
{
  AppendixObject *appendix;

  if (!PyArg_ParseTuple(args, "O!", &AppendixType, &appendix))
    return NULL;
  return PyInt_FromLong(eb_wide_alt_start(&appendix->appendix));
}

static char py_eb_wide_alt_end__doc__[] = "";
static PyObject *
py_eb_wide_alt_end(PyObject *self, PyObject *args)
{
  AppendixObject *appendix;

  if (!PyArg_ParseTuple(args, "O!", &AppendixType, &appendix))
    return NULL;
  return PyInt_FromLong(eb_wide_alt_end(&appendix->appendix));
}

static char py_eb_wide_alt_character_text__doc__[] = "";
static PyObject *
py_eb_wide_alt_character_text(PyObject *self, PyObject *args)
{
  AppendixObject *appendix;
  BufferObject *buffer;
  int ch, ret;
  char workbuf[EB_MAXLEN_TEXT_WORK + 1];

  if (!PyArg_ParseTuple(args, "O!iO!",
                        &AppendixType, &appendix, &ch, &BufferType, &buffer))
    return NULL;
  ret = eb_wide_alt_character_text(&appendix->appendix, ch, workbuf);
  if (ret >= 0) {
    Py_DECREF(buffer->buffer);
    buffer->buffer = PyString_FromString(workbuf);
  }
  return PyInt_FromLong(ret);
}

static char py_eb_forward_wide_alt_character__doc__[] = "";
static PyObject *
py_eb_forward_wide_alt_character(PyObject *self, PyObject *args)
{
  AppendixObject *appendix;
  int ch, n;

  if (!PyArg_ParseTuple(args, "O!ii", &AppendixType, &appendix, &ch, &n))
    return NULL;
  return PyInt_FromLong
    (eb_forward_wide_alt_character(&appendix->appendix, ch, n));
}

static char py_eb_backward_wide_alt_character__doc__[] = "";
static PyObject *
py_eb_backward_wide_alt_character(PyObject *self, PyObject *args)
{
  AppendixObject *appendix;
  int ch, n;

  if (!PyArg_ParseTuple(args, "O!ii", &AppendixType, &appendix, &ch, &n))
    return NULL;
  return PyInt_FromLong
    (eb_backward_wide_alt_character(&appendix->appendix, ch, n));
}

/* [text.h] */

/* hook.c */

#define EB_MAX_ARGV 4  /* eb-2.3.7 (text.c) */

static PyObject *
call_predefined_hook(int (*func)(EB_Book *, EB_Appendix *, char *,
                                 EB_Hook_Code, int, const int *),
                     PyObject *args)
{
  BookObject *book;
  AppendixObject *appendix;
  BufferObject *buffer;
  PyObject *argv, *element;
  EB_Hook_Code code;
  int i, ret, c_argc, c_argv[EB_MAX_ARGV];
  char workbuf[EB_MAXLEN_TEXT_WORK + 1] = "";

  if (!PyArg_ParseTuple(args, "O!O!O!iO!",
                        &BookType, &book, &AppendixType, &appendix,
                        &BufferType, &buffer, &code, &PyTuple_Type, &argv))
    return NULL;
  c_argc = PyTuple_Size(argv);
  for (i = 0; i < c_argc; i++) {
    element = PyTuple_GetItem(argv, i);
    if (!PyInt_Check(element)) {
      PyErr_SetString(PyExc_ValueError,
                      "4th argument must be a tuple of integers");
      return NULL;
    }
    c_argv[i] = PyInt_AsLong(element);
  }
  ret = func(&book->book, &appendix->appendix, workbuf, code, c_argc, c_argv);
  if (ret >= 0) {
    Py_DECREF(buffer->buffer);
    buffer->buffer = PyString_FromString(workbuf);
  }
  return PyInt_FromLong(ret);
}

static char py_eb_hook_euc_to_ascii__doc__[] = "";
static PyObject *
py_eb_hook_euc_to_ascii(PyObject *self, PyObject *args)
{
  return call_predefined_hook(eb_hook_euc_to_ascii, args);
}

static char py_eb_hook_stopcode__doc__[] = "";
static PyObject *
py_eb_hook_stopcode(PyObject *self, PyObject *args)
{
  return call_predefined_hook(eb_hook_stopcode, args);
}

static char py_eb_hook_stopcode_dummy__doc__[] = "";
static PyObject *
py_eb_hook_stopcode_dummy(PyObject *self, PyObject *args)
{
  return call_predefined_hook(eb_hook_stopcode_dummy, args);
}

static char py_eb_hook_stopcode_mixed__doc__[] = "";
static PyObject *
py_eb_hook_stopcode_mixed(PyObject *self, PyObject *args)
{
  return call_predefined_hook(eb_hook_stopcode_mixed, args);
}

static char py_eb_hook_narrow_character_text__doc__[] = "";
static PyObject *
py_eb_hook_narrow_character_text(PyObject *self, PyObject *args)
{
  return call_predefined_hook(eb_hook_narrow_character_text, args);
}

static char py_eb_hook_wide_character_text__doc__[] = "";
static PyObject *
py_eb_hook_wide_character_text(PyObject *self, PyObject *args)
{
  return call_predefined_hook(eb_hook_wide_character_text, args);
}

static char py_eb_hook_empty__doc__[] = "";
static PyObject *
py_eb_hook_empty(PyObject *self, PyObject *args)
{
  return call_predefined_hook(eb_hook_empty, args);
}

/* text.c */

static char py_eb_Hookset__doc__[] = "\
EB_Hookset() => hookset

Return a new instance of the Hookset object.";

static PyObject *
py_eb_Hookset(PyObject *self, PyObject *args)
{
  if (!PyArg_ParseTuple(args, ""))
    return NULL;
  return (PyObject *)hookset_new();
}

static int
hook_handler(EB_Book *c_book, EB_Appendix *c_appendix, char *c_buff,
	     EB_Hook_Code code, int c_argc, const int *c_argv)
{
  PyObject *context, *argv, *callback, *callback_args, *result;
  BookObject *book;
  AppendixObject *appendix;
  HooksetObject *hookset;
  BufferObject *buff;
  int i, ret;
  char *text;

  /* prepare callback arguments based on context information */
  context = PyDict_GetItem(CallbackContext, PyInt_FromLong(c_book->code));
  book     = (BookObject     *)PyTuple_GetItem(context, 0);
  appendix = (AppendixObject *)PyTuple_GetItem(context, 1);
  hookset  = (HooksetObject  *)PyTuple_GetItem(context, 2);
  buff = buffer_new();
  argv = PyTuple_New(c_argc);
  for (i = 0; i < c_argc; i++)
    PyTuple_SetItem(argv, i, PyInt_FromLong(c_argv[i]));
  callback_args = PyTuple_New(5);
  Py_INCREF(book);
  Py_INCREF(appendix);
  Py_INCREF(buff);
  PyTuple_SetItem(callback_args, 0, (PyObject *)book);
  PyTuple_SetItem(callback_args, 1, (PyObject *)appendix);
  PyTuple_SetItem(callback_args, 2, (PyObject *)buff);
  PyTuple_SetItem(callback_args, 3, PyInt_FromLong(code));
  PyTuple_SetItem(callback_args, 4, argv);
  /* run callback function */
  callback = PyDict_GetItem(hookset->callbacks, PyInt_FromLong(code));
  Py_INCREF(callback);
  result = PyObject_CallObject(callback, callback_args);
  Py_DECREF(callback);
  Py_DECREF(callback_args);
  /* check return values */
  if (result == NULL) {
    ret = -1;
  } else if (!PyInt_Check(result)) {
    PyErr_SetString(PyExc_ValueError, "hook function must return an integer");
    ret = -1;
  } else {
    ret = PyInt_AsLong(result);
    if (ret >= 0 && buff->buffer != Py_None) {
      text = PyString_AsString(buff->buffer);
      if (strlen(text) > EB_MAXLEN_TEXT_WORK) {
        PyErr_SetString(PyExc_ValueError, "returned string too long");
        ret = -1;
      } else {
        strcpy(c_buff, text);
      }
    }
  }
  if (PyErr_Occurred())
    PyErr_Print();
  Py_DECREF(buff);
  Py_XDECREF(result);
  return ret;
}

static char py_eb_set_hook__doc__[] = "\
eb_set_hook(hookset, (int, callable)) => int

Register a hook function.  Return -1 if failed.";

static PyObject *
py_eb_set_hook(PyObject *self, PyObject *args)
{
  HooksetObject *hookset;
  PyObject *callback;
  EB_Hook hook = {EB_HOOK_NULL, hook_handler};
  int ret;

  if (!PyArg_ParseTuple(args, "O!(iO)",
                        &HooksetType, &hookset, &hook.code, &callback))
    return NULL;
  if (!PyCallable_Check(callback)) {
    PyErr_SetString(PyExc_TypeError, "hook function must be callable");
    return NULL;
  }
  if ((ret = eb_set_hook(&hookset->hookset, &hook)) >= 0) {
    PyDict_SetItem(hookset->callbacks, PyInt_FromLong(hook.code), callback);
    Py_INCREF(callback);
  }
  return PyInt_FromLong(ret);
}

static char py_eb_set_hooks__doc__[] = "\
eb_set_hooks(hookset, (int, callable)[]) => int

Register a set of hook functions.  Return -1 if failed.";

static PyObject *
py_eb_set_hooks(PyObject *self, PyObject *args)
{
  HooksetObject *hookset;
  PyObject *tuple, *element, *callback;
  EB_Hook hooks[EB_NUM_HOOKS+1];
  int size, i, ret;

  if (!PyArg_ParseTuple(args, "O!O!",
                        &HooksetType, &hookset, &PyTuple_Type, &tuple))
    return NULL;
  size = PyTuple_Size(tuple);
  if (size > EB_NUM_HOOKS) {
    PyErr_SetString(PyExc_TypeError, "too many hooks");
    return NULL;
  }
  for (i = 0; i < size; i++) {
    element = PyTuple_GetItem(tuple, i);
    if (!PyTuple_Check(element) || PyTuple_Size(element) != 2) {
      PyErr_SetString(PyExc_TypeError,
                      "each hook must be a pair of a hook code and function");
      return NULL;
    }
    hooks[i].code = PyInt_AsLong(PyTuple_GetItem(element, 0));
    hooks[i].function = hook_handler;
    if (!PyCallable_Check(PyTuple_GetItem(element, 1))) {
      PyErr_SetString(PyExc_ValueError, "hook function must be callable");
      return NULL;
    }
  }
  hooks[i].code = EB_HOOK_NULL;
  hooks[i].function = NULL;
  if ((ret = eb_set_hooks(&hookset->hookset, hooks)) < 0)
    return PyInt_FromLong(ret);
  for (i = 0; i < size; i++) {
    callback = PyTuple_GetItem(PyTuple_GetItem(tuple, i), 1);
    PyDict_SetItem(hookset->callbacks, PyInt_FromLong(hooks[i].code),
                   callback);
    Py_INCREF(callback);
  }
  return PyInt_FromLong(ret);
}

static char py_eb_seek__doc__[] = "\
eb_seek(book, (int, int)) => int

Seek the file to the specified position.  Return -1 if failed.";

static PyObject *
py_eb_seek(PyObject *self, PyObject *args)
{
  BookObject *book;
  EB_Position pos;

  if (!PyArg_ParseTuple(args, "O!(ii)",
                        &BookType, &book, &pos.page, &pos.offset))
    return NULL;
  return PyInt_FromLong(eb_seek(&book->book, &pos));
}

static char py_eb_text__doc__[] = "\
eb_text(book, appendix, hookset) => string

Return the body of the section starting from the current file position.  Return None if failed.";

static PyObject *
py_eb_text(PyObject *self, PyObject *args)
{
  BookObject *book;
  AppendixObject *appendix;
  HooksetObject *hookset;
  PyObject *key, *context;
  char buffer[EB_SIZE_PAGE];
  int len;

  if (!PyArg_ParseTuple(args, "O!O!O!", &BookType, &book,
                        &AppendixType, &appendix, &HooksetType, &hookset))
    return NULL;
  key = PyInt_FromLong(book->book.code);
  context = Py_BuildValue("(OOO)", book, appendix, hookset);
  PyDict_SetItem(CallbackContext, key, context);
  len = eb_text(&book->book, &appendix->appendix, &hookset->hookset,
		buffer, EB_SIZE_PAGE-1);
  PyDict_DelItem(CallbackContext, key);
  Py_DECREF(key);
  Py_DECREF(context);
  if (len < 0) {
    Py_INCREF(Py_None);
    return Py_None;
  }
  return PyString_FromStringAndSize(buffer, len);
}

static char py_eb_heading__doc__[] = "\
eb_heading(book, appendix, hookset) => string

Return the heading of the section starting from the current file position.  Return None if failed.";

static PyObject *
py_eb_heading(PyObject *self, PyObject *args)
{
  BookObject *book;
  AppendixObject *appendix;
  HooksetObject *hookset;
  PyObject *key, *context;
  char buffer[EB_SIZE_PAGE];
  int len;

  if (!PyArg_ParseTuple(args, "O!O!O!", &BookType, &book,
                        &AppendixType, &appendix, &HooksetType, &hookset))
    return NULL;
  key = PyInt_FromLong(book->book.code);
  context = Py_BuildValue("(OOO)", book, appendix, hookset);
  PyDict_SetItem(CallbackContext, key, context);
  len = eb_heading(&book->book, &appendix->appendix, &hookset->hookset,
		   buffer, EB_SIZE_PAGE-1);
  PyDict_DelItem(CallbackContext, key);
  Py_DECREF(key);
  Py_DECREF(context);
  if (len < 0) {
    Py_INCREF(Py_None);
    return Py_None;
  }
  return PyString_FromStringAndSize(buffer, len);
}

static char py_eb_rawtext__doc__[] = "\
eb_rawtext(book, int) => string

Return the given number of raw characters read from the current file position.  Return None if failed.";

static PyObject *
py_eb_rawtext(PyObject *self, PyObject *args)
{
  BookObject *book;
  char *buffer;
  int size;
  PyObject *ret;

  if (!PyArg_ParseTuple(args, "O!i", &BookType, &book, &size))
    return NULL;
  if ((buffer = (char *)malloc(size + 1)) == NULL)
    return PyErr_NoMemory();
  if ((size = eb_rawtext(&book->book, buffer, size)) < 0) {
    Py_INCREF(Py_None);
    return Py_None;
  } else {
    ret = PyString_FromStringAndSize(buffer, size);
  }
  free(buffer);
  return ret;
}

/* unit.c */

/* [error.h] */

/* error.c */

static char py_eb_error_message__doc__[] = "\
eb_error_message() => string

Return the error message of the current error status.";

static PyObject *
py_eb_error_message(PyObject *self, PyObject *args)
{
  if (!PyArg_ParseTuple(args, ""))
    return NULL;
  return PyString_FromString(eb_error_message());
}

static char py_eb_error_message2__doc__[] = "\
eb_error_message2(int) => string

Return the error message of the specified error code.";

static PyObject *
py_eb_error_message2(PyObject *self, PyObject *args)
{
  int error_code;

  if (!PyArg_ParseTuple(args, "i", &error_code))
    return NULL;
  return PyString_FromString(eb_error_message2(error_code));
}

/* [language.h] */

/* language.c */

/* message.c */

/* [font.h] */

/* bitmap.c */

/* font.c */

/* narwfont.c */

/* widefont.c */

/* List of methods defined in the eb module */

#define meth(name, func, doc) {name, (PyCFunction)func, METH_VARARGS, doc}

static struct PyMethodDef eb_module_methods[] = {
  /* additional function */
  meth("Buffer",
       mod_Buffer,
       mod_Buffer__doc__),
  /* [eb.h] */
  /* copyright.c */
  meth("eb_have_copyright",
       py_eb_have_copyright,
       py_eb_have_copyright__doc__),
  meth("eb_copyright",
       py_eb_copyright,
       py_eb_copyright__doc__),
  /* eb.c */
  meth("EB_Book", /* eb_initialize(book) */
       py_eb_Book,
       py_eb_Book__doc__),
  meth("eb_bind",
       py_eb_bind,
       py_eb_bind__doc__),
  meth("eb_suspend",
       py_eb_suspend,
       py_eb_suspend__doc__),
  meth("eb_clear",
       py_eb_clear,
       py_eb_clear__doc__),
  meth("eb_is_bound",
       py_eb_is_bound,
       py_eb_is_bound__doc__),
  meth("eb_path",
       py_eb_path,
       py_eb_path__doc__),
  meth("eb_disc_type",
       py_eb_disc_type,
       py_eb_disc_type__doc__),
  meth("eb_character_code",
       py_eb_character_code,
       py_eb_character_code__doc__),
  /* filename.c */
  /* graphic.c */
  meth("eb_have_graphic_search",
       py_eb_have_graphic_search,
       py_eb_have_graphic_search__doc__),
  /* keyword.c */
  meth("eb_have_keyword_search",
       py_eb_have_keyword_search,
       py_eb_have_keyword_search__doc__),
  /* menu.c */
  meth("eb_have_menu",
       py_eb_have_menu,
       py_eb_have_menu__doc__),
  meth("eb_menu",
       py_eb_menu,
       py_eb_menu__doc__),
  /* multi.c */
  meth("eb_have_multi_search",
       py_eb_have_multi_search,
       py_eb_have_multi_search__doc__),
  /* search.c */
  meth("eb_have_exactword_search",
       py_eb_have_exactword_search,
       py_eb_have_exactword_search__doc__),
  meth("eb_search_exactword",
       py_eb_search_exactword,
       py_eb_search_exactword__doc__),
  meth("eb_have_word_search",
       py_eb_have_word_search,
       py_eb_have_word_search__doc__),
  meth("eb_search_word",
       py_eb_search_word,
       py_eb_search_word__doc__),
  meth("eb_have_endword_search",
       py_eb_have_endword_search,
       py_eb_have_endword_search__doc__),
  meth("eb_search_endword",
       py_eb_search_endword,
       py_eb_search_endword__doc__),
  meth("eb_hit_list",
       py_eb_hit_list,
       py_eb_hit_list__doc__),
  /* subbook.c */
  meth("eb_initialize_all_subbooks",
       py_eb_initialize_all_subbooks,
       py_eb_initialize_all_subbooks__doc__),
  meth("eb_subbook_count",
       py_eb_subbook_count,
       py_eb_subbook_count__doc__),
  meth("eb_subbook_list",
       py_eb_subbook_list,
       py_eb_subbook_list__doc__),
  meth("eb_subbook",
       py_eb_subbook,
       py_eb_subbook__doc__),
  meth("eb_subbook_title",
       py_eb_subbook_title,
       py_eb_subbook_title__doc__),
  meth("eb_subbook_title2",
       py_eb_subbook_title2,
       py_eb_subbook_title2__doc__),
  meth("eb_subbook_directory",
       py_eb_subbook_directory,
       py_eb_subbook_directory__doc__),
  meth("eb_subbook_directory2",
       py_eb_subbook_directory2,
       py_eb_subbook_directory2__doc__),
  meth("eb_set_subbook",
       py_eb_set_subbook,
       py_eb_set_subbook__doc__),
  meth("eb_unset_subbook",
       py_eb_unset_subbook,
       py_eb_unset_subbook__doc__),
  /* [appendix.h] */
  /* appendix.c */
  meth("EB_Appendix", /* eb_initialize_appendix(appendix) */
       py_eb_Appendix,
       py_eb_Appendix__doc__),
  meth("eb_clear_appendix",
       py_eb_clear_appendix,
       py_eb_clear_appendix__doc__),
  meth("eb_suspend_appendix",
       py_eb_suspend_appendix,
       py_eb_suspend_appendix__doc__),
  meth("eb_bind_appendix",
       py_eb_bind_appendix,
       py_eb_bind_appendix__doc__),
  meth("eb_is_appendix_bound",
       py_eb_is_appendix_bound,
       py_eb_is_appendix_bound__doc__),
  meth("eb_appendix_path",
       py_eb_appendix_path,
       py_eb_appendix_path__doc__),
  /* appsub.c */
  meth("eb_initialize_appendix_subbook",
       py_eb_initialize_appendix_subbook,
       py_eb_initialize_appendix_subbook__doc__),
  meth("eb_initialize_all_appendix_subbooks",
       py_eb_initialize_all_appendix_subbooks,
       py_eb_initialize_all_appendix_subbooks__doc__),
  meth("eb_appendix_subbook_count",
       py_eb_appendix_subbook_count,
       py_eb_appendix_subbook_count__doc__),
  meth("eb_appendix_subbook_list",
       py_eb_appendix_subbook_list,
       py_eb_appendix_subbook_list__doc__),
  meth("eb_appendix_subbook",
       py_eb_appendix_subbook,
       py_eb_appendix_subbook__doc__),
  meth("eb_appendix_subbook2",
       py_eb_appendix_subbook2,
       py_eb_appendix_subbook2__doc__),
  meth("eb_appendix_subbook_directory",
       py_eb_appendix_subbook_directory,
       py_eb_appendix_subbook_directory__doc__),
  meth("eb_appendix_subbook_directory2",
       py_eb_appendix_subbook_directory2,
       py_eb_appendix_subbook_directory2__doc__),
  meth("eb_set_appendix_subbook",
       py_eb_set_appendix_subbook,
       py_eb_set_appendix_subbook__doc__),
  meth("eb_unset_appendix_subbook",
       py_eb_unset_appendix_subbook,
       py_eb_unset_appendix_subbook__doc__),
  /* stopcode.c */
  meth("eb_have_stopcode",
       py_eb_have_stopcode,
       py_eb_have_stopcode__doc__),
  meth("eb_stopcode",
       py_eb_stopcode,
       py_eb_stopcode__doc__),
  /* narwalt.c */
  meth("eb_have_narrow_alt",
       py_eb_have_narrow_alt,
       py_eb_have_narrow_alt__doc__),
  meth("eb_narrow_alt_start",
       py_eb_narrow_alt_start,
       py_eb_narrow_alt_start__doc__),
  meth("eb_narrow_alt_end",
       py_eb_narrow_alt_end,
       py_eb_narrow_alt_end__doc__),
  meth("eb_narrow_alt_character_text",
       py_eb_narrow_alt_character_text,
       py_eb_narrow_alt_character_text__doc__),
  meth("eb_forward_narrow_alt_character",
       py_eb_forward_narrow_alt_character,
       py_eb_forward_narrow_alt_character__doc__),
  meth("eb_backward_narrow_alt_character",
       py_eb_backward_narrow_alt_character,
       py_eb_backward_narrow_alt_character__doc__),
  /* widealt.c */
  meth("eb_have_wide_alt",
       py_eb_have_wide_alt,
       py_eb_have_wide_alt__doc__),
  meth("eb_wide_alt_start",
       py_eb_wide_alt_start,
       py_eb_wide_alt_start__doc__),
  meth("eb_wide_alt_end",
       py_eb_wide_alt_end,
       py_eb_wide_alt_end__doc__),
  meth("eb_wide_alt_character_text",
       py_eb_wide_alt_character_text,
       py_eb_wide_alt_character_text__doc__),
  meth("eb_forward_wide_alt_character",
       py_eb_forward_wide_alt_character,
       py_eb_forward_wide_alt_character__doc__),
  meth("eb_backward_wide_alt_character",
       py_eb_backward_wide_alt_character,
       py_eb_backward_wide_alt_character__doc__),
  /* [text.h] */
  /* hook.c */
  meth("eb_hook_euc_to_ascii",
       py_eb_hook_euc_to_ascii,
       py_eb_hook_euc_to_ascii__doc__),
  meth("eb_hook_stopcode",
       py_eb_hook_stopcode,
       py_eb_hook_stopcode__doc__),
  meth("eb_hook_stopcode_dummy",
       py_eb_hook_stopcode_dummy,
       py_eb_hook_stopcode_dummy__doc__),
  meth("eb_hook_stopcode_mixed",
       py_eb_hook_stopcode_mixed,
       py_eb_hook_stopcode_mixed__doc__),
  meth("eb_hook_narrow_character_text",
       py_eb_hook_narrow_character_text,
       py_eb_hook_narrow_character_text__doc__),
  meth("eb_hook_wide_character_text",
       py_eb_hook_wide_character_text,
       py_eb_hook_wide_character_text__doc__),
  meth("eb_hook_empty",
       py_eb_hook_empty,
       py_eb_hook_empty__doc__),
  /* text.c */
  meth("EB_Hookset", /* eb_initialize_hookset(hookset) */
       py_eb_Hookset,
       py_eb_Hookset__doc__),
  meth("eb_set_hook",
       py_eb_set_hook,
       py_eb_set_hook__doc__),
  meth("eb_set_hooks",
       py_eb_set_hooks,
       py_eb_set_hooks__doc__),
  meth("eb_seek",
       py_eb_seek,
       py_eb_seek__doc__),
  meth("eb_text",
       py_eb_text,
       py_eb_text__doc__),
  meth("eb_heading",
       py_eb_heading,
       py_eb_heading__doc__),
  meth("eb_rawtext",
       py_eb_rawtext,
       py_eb_rawtext__doc__),
  /* uint.c */
  /* [error.h] */
  /* error.c */
  meth("eb_error_message",
       py_eb_error_message,
       py_eb_error_message__doc__),
  meth("eb_error_message2",
       py_eb_error_message2,
       py_eb_error_message2__doc__),
  /* [language.h] */
  /* language.c */
  /* message.c */
  /* [font.h] */
  /* bitmap.c */
  /* font.c */
  /* narwfont.c */
  /* widefont.c */
  {NULL, (PyCFunction)NULL, 0, NULL} /* sentinel */
};

/* Initialization function for the module (*must* be called initeb) */

static char eb_module_documentation[] = "";

void
initeb(void)
{
  PyObject *m, *d;

  /* Create the module and add the functions */
  m = Py_InitModule4("eb", eb_module_methods, eb_module_documentation,
		     (PyObject*)NULL, PYTHON_API_VERSION);

  /* Add some symbolic constants to the module */
  d = PyModule_GetDict(m);

  EBError = PyErr_NewException("eb.EBError", NULL, NULL);
  PyDict_SetItemString(d, "EBError", EBError);

  CallbackContext = PyDict_New();
  PyDict_SetItemString(d, "_callback_context", CallbackContext);

#define constI(dict, name, value) \
        PyDict_SetItemString(dict, name, PyInt_FromLong(value));
#define constS(dict, name, value) \
        PyDict_SetItemString(dict, name, PyString_FromString(value));

  /* [defs.h] */
  /* Library version */
  constI(d, "EB_VERSION_MAJOR",		  EB_VERSION_MAJOR);
  constI(d, "EB_VERSION_MINOR",		  EB_VERSION_MINOR);
  /* Disc code */
  constI(d, "EB_DISC_EB",		  EB_DISC_EB);
  constI(d, "EB_DISC_EPWING",		  EB_DISC_EPWING);
  /* Case of filenames */
  constI(d, "EB_CASE_UPPER",		  EB_CASE_UPPER);
  constI(d, "EB_CASE_LOWER",		  EB_CASE_LOWER);
  /* Suffix to be added to filenames */
  constI(d, "EB_SUFFIX_NONE",		  EB_SUFFIX_NONE);
  constI(d, "EB_SUFFIX_DOT",		  EB_SUFFIX_DOT);
  constI(d, "EB_SUFFIX_PERIOD",		  EB_SUFFIX_PERIOD);
  constI(d, "EB_SUFFIX_VERSION",	  EB_SUFFIX_VERSION);
  constI(d, "EB_SUFFIX_BOTH",		  EB_SUFFIX_BOTH);
  /* Character codes */
  constI(d, "EB_CHARCODE_ISO8859_1",	  EB_CHARCODE_ISO8859_1);
  constI(d, "EB_CHARCODE_JISX0208",	  EB_CHARCODE_JISX0208);
  constI(d, "EB_CHARCODE_JISX0208_GB2312",EB_CHARCODE_JISX0208_GB2312);
  /* Search word types */
  constI(d, "EB_WORD_ALPHA",		  EB_WORD_ALPHA);
  constI(d, "EB_WORD_KANA",		  EB_WORD_KANA);
  constI(d, "EB_WORD_OTHER",		  EB_WORD_OTHER);
  constI(d, "EB_WORD_ERROR",		  EB_WORD_ERROR);
  /* Index Style flags */
  constI(d, "EB_INDEX_STYLE_CONVERT",	  EB_INDEX_STYLE_CONVERT);
  constI(d, "EB_INDEX_STYLE_ASIS",	  EB_INDEX_STYLE_ASIS);
  constI(d, "EB_INDEX_STYLE_DELETE",	  EB_INDEX_STYLE_DELETE);
  /* Compression type codes */
  constI(d, "EB_ZIP_EPWING",		  EB_ZIP_EPWING);
  constI(d, "EB_ZIP_NONE",		  EB_ZIP_NONE);
  constI(d, "EB_ZIP_EBZIP1",		  EB_ZIP_EBZIP1);
  /* Size and limitation */
  constI(d, "EB_SIZE_PAGE",		  EB_SIZE_PAGE);
  constI(d, "EB_SIZE_EB_CATALOG",	  EB_SIZE_EB_CATALOG);
  constI(d, "EB_SIZE_EPWING_CATALOG",	  EB_SIZE_EPWING_CATALOG);
  constI(d, "EB_SIZE_EBZIP_HEADER",	  EB_SIZE_EBZIP_HEADER);
  constI(d, "EB_SIZE_EBZIP_MARGIN",	  EB_SIZE_EBZIP_MARGIN);
  constI(d, "EB_MAXLEN_WORD",		  EB_MAXLEN_WORD);
  constI(d, "EB_MAXLEN_EB_TITLE",	  EB_MAXLEN_EB_TITLE);
  constI(d, "EB_MAXLEN_EPWING_TITLE",	  EB_MAXLEN_EPWING_TITLE);
  constI(d, "EB_MAXLEN_TITLE",		  EB_MAXLEN_TITLE);
  constI(d, "EB_MAXLEN_BASENAME",	  EB_MAXLEN_BASENAME);
  constI(d, "EB_MAXLEN_LANGNAME",	  EB_MAXLEN_LANGNAME);
  constI(d, "EB_MAXLEN_MESSAGE",	  EB_MAXLEN_MESSAGE);
  constI(d, "EB_MAXLEN_MULTI_LABEL",	  EB_MAXLEN_MULTI_LABEL);
  constI(d, "EB_MAXLEN_ALTERNATION_TEXT", EB_MAXLEN_ALTERNATION_TEXT);
  constI(d, "EB_MAX_FONTS",		  EB_MAX_FONTS);
  constI(d, "EB_MAX_SEARCHES",		  EB_MAX_SEARCHES);
  constI(d, "EB_MAX_SUBBOOKS",		  EB_MAX_SUBBOOKS);
  constI(d, "EB_MAX_LANGUAGES",		  EB_MAX_LANGUAGES);
  constI(d, "EB_MAX_MESSAGES",		  EB_MAX_MESSAGES);
  constI(d, "EB_MAX_INDEX_DEPTH",	  EB_MAX_INDEX_DEPTH);
  constI(d, "EB_MAX_KEYWORDS",		  EB_MAX_KEYWORDS);
  constI(d, "EB_MAX_MULTI_SEARCHES",	  EB_MAX_MULTI_SEARCHES);
  constI(d, "EB_MAX_MULTI_ENTRIES",	  EB_MAX_MULTI_ENTRIES);
  constI(d, "EB_MAX_ALTERNATION_CACHE",	  EB_MAX_ALTERNATION_CACHE);
  constI(d, "EB_MAX_EBZIP_LEVEL",	  EB_MAX_EBZIP_LEVEL);
  /* File and directory names */
  constS(d, "EB_FILENAME_START",	  EB_FILENAME_START);
  constS(d, "EB_FILENAME_SOUND",	  EB_FILENAME_SOUND);
  constS(d, "EB_FILENAME_CATALOG",	  EB_FILENAME_CATALOG);
  constS(d, "EB_FILENAME_LANGUAGE",	  EB_FILENAME_LANGUAGE);
  constS(d, "EB_FILENAME_VTOC",		  EB_FILENAME_VTOC);
  constS(d, "EB_FILENAME_WELCOME",	  EB_FILENAME_WELCOME);
  constS(d, "EB_FILENAME_CATALOGS",	  EB_FILENAME_CATALOGS);
  constS(d, "EB_FILENAME_HONMON",	  EB_FILENAME_HONMON);
  constS(d, "EB_FILENAME_HONMON2",	  EB_FILENAME_HONMON2);
  constS(d, "EB_FILENAME_APPENDIX",	  EB_FILENAME_APPENDIX);
  constS(d, "EB_FILENAME_FUROKU",	  EB_FILENAME_FUROKU);
  constS(d, "EB_DIRNAME_DATA",		  EB_DIRNAME_DATA);
  constS(d, "EB_DIRNAME_GAIJI",		  EB_DIRNAME_GAIJI);
  /* [text.h] */
  /* Hook codes */
  constI(d, "EB_HOOK_NULL",               EB_HOOK_NULL);
  constI(d, "EB_HOOK_BEGIN_NARROW",       EB_HOOK_BEGIN_NARROW);
  constI(d, "EB_HOOK_NARROW",             EB_HOOK_NARROW);
  constI(d, "EB_HOOK_END_NARROW",         EB_HOOK_END_NARROW);
  constI(d, "EB_HOOK_BEGIN_SUBSCRIPT",    EB_HOOK_BEGIN_SUBSCRIPT);
  constI(d, "EB_HOOK_SUBSCRIPT",          EB_HOOK_SUBSCRIPT);
  constI(d, "EB_HOOK_END_SUBSCRIPT",      EB_HOOK_END_SUBSCRIPT);
  constI(d, "EB_HOOK_SET_INDENT",         EB_HOOK_SET_INDENT);
  constI(d, "EB_HOOK_NEWLINE",            EB_HOOK_NEWLINE);
  constI(d, "EB_HOOK_BEGIN_SUPERSCRIPT",  EB_HOOK_BEGIN_SUPERSCRIPT);
  constI(d, "EB_HOOK_SUPERSCRIPT",        EB_HOOK_SUPERSCRIPT);
  constI(d, "EB_HOOK_END_SUPERSCRIPT",    EB_HOOK_END_SUPERSCRIPT);
  constI(d, "EB_HOOK_BEGIN_TABLE",        EB_HOOK_BEGIN_TABLE);
  constI(d, "EB_HOOK_END_TABLE",          EB_HOOK_END_TABLE);
  constI(d, "EB_HOOK_BEGIN_NO_NEWLINE",   EB_HOOK_BEGIN_NO_NEWLINE);
  constI(d, "EB_HOOK_NO_NEWLINE",         EB_HOOK_NO_NEWLINE);
  constI(d, "EB_HOOK_END_NO_NEWLINE",     EB_HOOK_END_NO_NEWLINE);
  constI(d, "EB_HOOK_BEGIN_EMPHASIS",     EB_HOOK_BEGIN_EMPHASIS);
  constI(d, "EB_HOOK_EMPHASIS",           EB_HOOK_EMPHASIS);
  constI(d, "EB_HOOK_END_EMPHASIS",       EB_HOOK_END_EMPHASIS);
  constI(d, "EB_HOOK_BEGIN_PICTURE",      EB_HOOK_BEGIN_PICTURE);
  constI(d, "EB_HOOK_PICTURE",            EB_HOOK_PICTURE);
  constI(d, "EB_HOOK_END_PICTURE",        EB_HOOK_END_PICTURE);
  constI(d, "EB_HOOK_BEGIN_MENU",         EB_HOOK_BEGIN_MENU);
  constI(d, "EB_HOOK_MENU",               EB_HOOK_MENU);
  constI(d, "EB_HOOK_END_MENU",           EB_HOOK_END_MENU);
  constI(d, "EB_HOOK_BEGIN_SOUND",        EB_HOOK_BEGIN_SOUND);
  constI(d, "EB_HOOK_SOUND",              EB_HOOK_SOUND);
  constI(d, "EB_HOOK_END_SOUND",          EB_HOOK_END_SOUND);
  constI(d, "EB_HOOK_BEGIN_REFERENCE",    EB_HOOK_BEGIN_REFERENCE);
  constI(d, "EB_HOOK_REFERENCE",          EB_HOOK_REFERENCE);
  constI(d, "EB_HOOK_END_REFERENCE",      EB_HOOK_END_REFERENCE);
  constI(d, "EB_HOOK_BEGIN_KEYWORD",      EB_HOOK_BEGIN_KEYWORD);
  constI(d, "EB_HOOK_KEYWORD",            EB_HOOK_KEYWORD);
  constI(d, "EB_HOOK_END_KEYWORD",        EB_HOOK_END_KEYWORD);
  constI(d, "EB_HOOK_ISO8859_1",          EB_HOOK_ISO8859_1);
  constI(d, "EB_HOOK_NARROW_JISX0208",    EB_HOOK_NARROW_JISX0208);
  constI(d, "EB_HOOK_WIDE_JISX0208",      EB_HOOK_WIDE_JISX0208);
  constI(d, "EB_HOOK_NARROW_FONT",        EB_HOOK_NARROW_FONT);
  constI(d, "EB_HOOK_WIDE_FONT",          EB_HOOK_WIDE_FONT);
  constI(d, "EB_HOOK_INITIALIZE",         EB_HOOK_INITIALIZE);
  constI(d, "EB_HOOK_STOPCODE",           EB_HOOK_STOPCODE);
  constI(d, "EB_HOOK_GB2312",             EB_HOOK_GB2312);
  constI(d, "EB_NUM_HOOKS",               EB_NUM_HOOKS);
  constI(d, "EB_MAXLEN_TEXT_WORK",        EB_MAXLEN_TEXT_WORK);
  /* [error.h] */
  /* Error codes */
  constI(d, "EB_NO_ERR",                  EB_NO_ERR);
  constI(d, "EB_ERR_MEMORY_EXHAUSTED",    EB_ERR_MEMORY_EXHAUSTED);
  constI(d, "EB_ERR_EMPTY_FILENAME",      EB_ERR_EMPTY_FILENAME);
  constI(d, "EB_ERR_TOO_LONG_FILENAME",   EB_ERR_TOO_LONG_FILENAME);
  constI(d, "EB_ERR_TOO_LONG_WORD",       EB_ERR_TOO_LONG_WORD);
  constI(d, "EB_ERR_BAD_WORD",            EB_ERR_BAD_WORD);
  constI(d, "EB_ERR_EMPTY_WORD",          EB_ERR_EMPTY_WORD);
  constI(d, "EB_ERR_FAIL_GETCWD",         EB_ERR_FAIL_GETCWD);
  constI(d, "EB_ERR_FAIL_OPEN_CAT",       EB_ERR_FAIL_OPEN_CAT);
  constI(d, "EB_ERR_FAIL_OPEN_CATAPP",    EB_ERR_FAIL_OPEN_CATAPP);
  constI(d, "EB_ERR_FAIL_OPEN_LANG",      EB_ERR_FAIL_OPEN_LANG);
  constI(d, "EB_ERR_FAIL_OPEN_START",     EB_ERR_FAIL_OPEN_START);
  constI(d, "EB_ERR_FAIL_OPEN_FONT",      EB_ERR_FAIL_OPEN_FONT);
  constI(d, "EB_ERR_FAIL_OPEN_APP",       EB_ERR_FAIL_OPEN_APP);
  constI(d, "EB_ERR_FAIL_READ_CAT",       EB_ERR_FAIL_READ_CAT);
  constI(d, "EB_ERR_FAIL_READ_CATAPP",    EB_ERR_FAIL_READ_CATAPP);
  constI(d, "EB_ERR_FAIL_READ_LANG",      EB_ERR_FAIL_READ_LANG);
  constI(d, "EB_ERR_FAIL_READ_START",     EB_ERR_FAIL_READ_START);
  constI(d, "EB_ERR_FAIL_READ_FONT",      EB_ERR_FAIL_READ_FONT);
  constI(d, "EB_ERR_FAIL_READ_APP",       EB_ERR_FAIL_READ_APP);
  constI(d, "EB_ERR_FAIL_SEEK_CAT",       EB_ERR_FAIL_SEEK_CAT);
  constI(d, "EB_ERR_FAIL_SEEK_CATAPP",    EB_ERR_FAIL_SEEK_CATAPP);
  constI(d, "EB_ERR_FAIL_SEEK_LANG",      EB_ERR_FAIL_SEEK_LANG);
  constI(d, "EB_ERR_FAIL_SEEK_START",     EB_ERR_FAIL_SEEK_START);
  constI(d, "EB_ERR_FAIL_SEEK_FONT",      EB_ERR_FAIL_SEEK_FONT);
  constI(d, "EB_ERR_FAIL_SEEK_APP",       EB_ERR_FAIL_SEEK_APP);
  constI(d, "EB_ERR_UNEXP_CAT",           EB_ERR_UNEXP_CAT);
  constI(d, "EB_ERR_UNEXP_CATAPP",        EB_ERR_UNEXP_CATAPP);
  constI(d, "EB_ERR_UNEXP_LANG",          EB_ERR_UNEXP_LANG);
  constI(d, "EB_ERR_UNEXP_START",         EB_ERR_UNEXP_START);
  constI(d, "EB_ERR_UNEXP_FONT",          EB_ERR_UNEXP_FONT);
  constI(d, "EB_ERR_UNEXP_APP",           EB_ERR_UNEXP_APP);
  constI(d, "EB_ERR_UNBOUND_BOOK",        EB_ERR_UNBOUND_BOOK);
  constI(d, "EB_ERR_UNBOUND_APP",         EB_ERR_UNBOUND_APP);
  constI(d, "EB_ERR_NO_LANG",             EB_ERR_NO_LANG);
  constI(d, "EB_ERR_NO_SUB",              EB_ERR_NO_SUB);
  constI(d, "EB_ERR_NO_APPSUB",           EB_ERR_NO_APPSUB);
  constI(d, "EB_ERR_NO_MSG",              EB_ERR_NO_MSG);
  constI(d, "EB_ERR_NO_FONT",             EB_ERR_NO_FONT);
  constI(d, "EB_ERR_NO_START",            EB_ERR_NO_START);
  constI(d, "EB_ERR_NO_CUR_LANG",         EB_ERR_NO_CUR_LANG);
  constI(d, "EB_ERR_NO_CUR_SUB",          EB_ERR_NO_CUR_SUB);
  constI(d, "EB_ERR_NO_CUR_APPSUB",       EB_ERR_NO_CUR_APPSUB);
  constI(d, "EB_ERR_NO_CUR_FONT",         EB_ERR_NO_CUR_FONT);
  constI(d, "EB_ERR_NO_SUCH_LANG",        EB_ERR_NO_SUCH_LANG);
  constI(d, "EB_ERR_NO_SUCH_SUB",         EB_ERR_NO_SUCH_SUB);
  constI(d, "EB_ERR_NO_SUCH_APPSUB",      EB_ERR_NO_SUCH_APPSUB);
  constI(d, "EB_ERR_NO_SUCH_MSG",         EB_ERR_NO_SUCH_MSG);
  constI(d, "EB_ERR_NO_SUCH_FONT",        EB_ERR_NO_SUCH_FONT);
  constI(d, "EB_ERR_NO_SUCH_CHAR_BMP",    EB_ERR_NO_SUCH_CHAR_BMP);
  constI(d, "EB_ERR_NO_SUCH_CHAR_TEXT",   EB_ERR_NO_SUCH_CHAR_TEXT);
  constI(d, "EB_ERR_NO_SUCH_SEARCH",      EB_ERR_NO_SUCH_SEARCH);
  constI(d, "EB_ERR_NO_SUCH_HOOK",        EB_ERR_NO_SUCH_HOOK);
  constI(d, "EB_ERR_HOOK_WORKSPACE",      EB_ERR_HOOK_WORKSPACE);
  constI(d, "EB_ERR_DIFF_CONTENT",        EB_ERR_DIFF_CONTENT);
  constI(d, "EB_ERR_DIFF_SUBBOOK",        EB_ERR_DIFF_SUBBOOK);
  constI(d, "EB_ERR_DIFF_BOOK",           EB_ERR_DIFF_BOOK);
  constI(d, "EB_ERR_NO_PREV_SEARCH",      EB_ERR_NO_PREV_SEARCH);
  constI(d, "EB_ERR_NO_PREV_CONTENT",     EB_ERR_NO_PREV_CONTENT);
  constI(d, "EB_ERR_NO_SUCH_MULTI_ID",    EB_ERR_NO_SUCH_MULTI_ID);
  constI(d, "EB_ERR_NO_SUCH_ENTRY_ID",    EB_ERR_NO_SUCH_ENTRY_ID);
  constI(d, "EB_NUM_ERRORS",              EB_NUM_ERRORS);
  constI(d, "EB_MAXLEN_ERROR_MESSAGE",    EB_MAXLEN_ERROR_MESSAGE);
  /* [language.h] */
  /* Language codes */
  constI(d, "EB_LANG_ENGLISH",            EB_LANG_ENGLISH);
  constI(d, "EB_LANG_FRENCH",             EB_LANG_FRENCH);
  constI(d, "EB_LANG_GERMAN",             EB_LANG_GERMAN);
  constI(d, "EB_LANG_ITALIAN",            EB_LANG_ITALIAN);
  constI(d, "EB_LANG_SPANISH",            EB_LANG_SPANISH);
  constI(d, "EB_LANG_JAPANESE",           EB_LANG_JAPANESE);
  constI(d, "EB_LANG_DANISH",             EB_LANG_DANISH);
  constI(d, "EB_LANG_FINNISH",            EB_LANG_FINNISH);
  constI(d, "EB_LANG_SWEDISH",            EB_LANG_SWEDISH);
  constI(d, "EB_LANG_NORWEGIAN",          EB_LANG_NORWEGIAN);
  constI(d, "EB_LANG_DUTCH",              EB_LANG_DUTCH);
  constI(d, "EB_LANG_POLISH",             EB_LANG_POLISH);
  /* Message codes */
  constI(d, "EB_MSG_WORD_SEARCH",         EB_MSG_WORD_SEARCH);
  constI(d, "EB_MSG_END_SEARCH",          EB_MSG_END_SEARCH);
  constI(d, "EB_MSG_KEY_SEARCH",          EB_MSG_KEY_SEARCH);
  constI(d, "EB_MSG_MENU_SEARCH",         EB_MSG_MENU_SEARCH);
  constI(d, "EB_MSG_MULTI_SEARCH",        EB_MSG_MULTI_SEARCH);
  constI(d, "EB_MSG_GRAPHIC_SEARCH",      EB_MSG_GRAPHIC_SEARCH);
  constI(d, "EB_MSG_BOOK_CONTAIN",        EB_MSG_BOOK_CONTAIN);
  constI(d, "EB_MSG_ENTER_WORD",          EB_MSG_ENTER_WORD);
  constI(d, "EB_MSG_ENTER_WORDS",         EB_MSG_ENTER_WORDS);
  constI(d, "EB_MSG_SUCCESSFUL",          EB_MSG_SUCCESSFUL);
  constI(d, "EB_MSG_ENTRY",               EB_MSG_ENTRY);
  constI(d, "EB_MSG_ENTRIES",             EB_MSG_ENTRIES);
  constI(d, "EB_MSG_SEARCH_FAIL",         EB_MSG_SEARCH_FAIL);
  constI(d, "EB_MSG_PUSH_NO",             EB_MSG_PUSH_NO);
  constI(d, "EB_MSG_INSERT_DISC",         EB_MSG_INSERT_DISC);
  constI(d, "EB_MSG_CANT_READ",           EB_MSG_CANT_READ);
  constI(d, "EB_MSG_REINSERT_DISC",       EB_MSG_REINSERT_DISC);
  constI(d, "EB_MSG_CLEAN_DISC",          EB_MSG_CLEAN_DISC);
  constI(d, "EB_MSG_NOT_EB_DISC",         EB_MSG_NOT_EB_DISC);
  constI(d, "EB_MSG_SEARCHING",           EB_MSG_SEARCHING);
  constI(d, "EB_MSG_CHANGING_BATT",       EB_MSG_CHANGING_BATT);
  constI(d, "EB_MSG_DRY_BATT",            EB_MSG_DRY_BATT);
  constI(d, "EB_MSG_CHARGE_END",          EB_MSG_CHARGE_END);
  constI(d, "EB_MSG_BATT_EMPTY",          EB_MSG_BATT_EMPTY);
  constI(d, "EB_MSG_BATT_CHANGE",         EB_MSG_BATT_CHANGE);
  constI(d, "EB_MSG_CANT_CHANGE_BATT",    EB_MSG_CANT_CHANGE_BATT);
  constI(d, "EB_MSG_SELECT_LANGUAGE",     EB_MSG_SELECT_LANGUAGE);
  constI(d, "EB_MSG_MENU",                EB_MSG_MENU);
  /* [font.h] */
  /* Font types */
  constI(d, "EB_FONT_16",                 EB_FONT_16);
  constI(d, "EB_FONT_24",                 EB_FONT_24);
  constI(d, "EB_FONT_30",                 EB_FONT_30);
  constI(d, "EB_FONT_48",                 EB_FONT_48);
  /* Font sizes */
  constI(d, "EB_SIZE_NARROW_FONT_16",	  EB_SIZE_NARROW_FONT_16);
  constI(d, "EB_SIZE_WIDE_FONT_16",	  EB_SIZE_WIDE_FONT_16);
  constI(d, "EB_SIZE_NARROW_FONT_24",	  EB_SIZE_NARROW_FONT_24);
  constI(d, "EB_SIZE_WIDE_FONT_24",	  EB_SIZE_WIDE_FONT_24);
  constI(d, "EB_SIZE_NARROW_FONT_30",	  EB_SIZE_NARROW_FONT_30);
  constI(d, "EB_SIZE_WIDE_FONT_30",	  EB_SIZE_WIDE_FONT_30);
  constI(d, "EB_SIZE_NARROW_FONT_48",	  EB_SIZE_NARROW_FONT_48);
  constI(d, "EB_SIZE_WIDE_FONT_48",	  EB_SIZE_WIDE_FONT_48);
  /* Font width */
  constI(d, "EB_WIDTH_NARROW_FONT_16",	  EB_WIDTH_NARROW_FONT_16);
  constI(d, "EB_WIDTH_WIDE_FONT_16",	  EB_WIDTH_WIDE_FONT_16);
  constI(d, "EB_WIDTH_NARROW_FONT_24",	  EB_WIDTH_NARROW_FONT_24);
  constI(d, "EB_WIDTH_WIDE_FONT_24",	  EB_WIDTH_WIDE_FONT_24);
  constI(d, "EB_WIDTH_NARROW_FONT_30",	  EB_WIDTH_NARROW_FONT_30);
  constI(d, "EB_WIDTH_WIDE_FONT_30",	  EB_WIDTH_WIDE_FONT_30);
  constI(d, "EB_WIDTH_NARROW_FONT_48",	  EB_WIDTH_NARROW_FONT_48);
  constI(d, "EB_WIDTH_WIDE_FONT_48",	  EB_WIDTH_WIDE_FONT_48);
  /* Bitmap image sizes */
  constI(d, "EB_SIZE_NARROW_FONT_16_XBM", EB_SIZE_NARROW_FONT_16_XBM);
  constI(d, "EB_SIZE_WIDE_FONT_16_XBM",	  EB_SIZE_WIDE_FONT_16_XBM);
  constI(d, "EB_SIZE_NARROW_FONT_16_XPM", EB_SIZE_NARROW_FONT_16_XPM);
  constI(d, "EB_SIZE_WIDE_FONT_16_XPM",	  EB_SIZE_WIDE_FONT_16_XPM);
  constI(d, "EB_SIZE_NARROW_FONT_16_GIF", EB_SIZE_NARROW_FONT_16_GIF);
  constI(d, "EB_SIZE_WIDE_FONT_16_GIF",	  EB_SIZE_WIDE_FONT_16_GIF);
  constI(d, "EB_SIZE_NARROW_FONT_24_XBM", EB_SIZE_NARROW_FONT_24_XBM);
  constI(d, "EB_SIZE_WIDE_FONT_24_XBM",	  EB_SIZE_WIDE_FONT_24_XBM);
  constI(d, "EB_SIZE_NARROW_FONT_24_XPM", EB_SIZE_NARROW_FONT_24_XPM);
  constI(d, "EB_SIZE_WIDE_FONT_24_XPM",	  EB_SIZE_WIDE_FONT_24_XPM);
  constI(d, "EB_SIZE_NARROW_FONT_24_GIF", EB_SIZE_NARROW_FONT_24_GIF);
  constI(d, "EB_SIZE_WIDE_FONT_24_GIF",	  EB_SIZE_WIDE_FONT_24_GIF);
  constI(d, "EB_SIZE_NARROW_FONT_30_XBM", EB_SIZE_NARROW_FONT_30_XBM);
  constI(d, "EB_SIZE_WIDE_FONT_30_XBM",	  EB_SIZE_WIDE_FONT_30_XBM);
  constI(d, "EB_SIZE_NARROW_FONT_30_XPM", EB_SIZE_NARROW_FONT_30_XPM);
  constI(d, "EB_SIZE_WIDE_FONT_30_XPM",	  EB_SIZE_WIDE_FONT_30_XPM);
  constI(d, "EB_SIZE_NARROW_FONT_30_GIF", EB_SIZE_NARROW_FONT_30_GIF);
  constI(d, "EB_SIZE_WIDE_FONT_30_GIF",	  EB_SIZE_WIDE_FONT_30_GIF);
  constI(d, "EB_SIZE_NARROW_FONT_48_XBM", EB_SIZE_NARROW_FONT_48_XBM);
  constI(d, "EB_SIZE_WIDE_FONT_48_XBM",	  EB_SIZE_WIDE_FONT_48_XBM);
  constI(d, "EB_SIZE_NARROW_FONT_48_XPM", EB_SIZE_NARROW_FONT_48_XPM);
  constI(d, "EB_SIZE_WIDE_FONT_48_XPM",	  EB_SIZE_WIDE_FONT_48_XPM);
  constI(d, "EB_SIZE_NARROW_FONT_48_GIF", EB_SIZE_NARROW_FONT_48_GIF);
  constI(d, "EB_SIZE_WIDE_FONT_48_GIF",	  EB_SIZE_WIDE_FONT_48_GIF);
  constI(d, "EB_SIZE_FONT_IMAGE",	  EB_SIZE_FONT_IMAGE);

  /* Check for errors */
  if (PyErr_Occurred())
    Py_FatalError("can't initialize the eb module");
}

/*

IMPLEMENTATION STATUS
'o' = implemented
'-' = not implemented
'n' = not available at the Python layer

[eb.h]
copyright.c:
o	int eb_have_copyright(book) 
o 	(int,int) eb_copyright(book, position)
eb.c:
n	void eb_initialize(book)
o	int eb_bind(book, string)
o	void eb_suspend(book)
o	void eb_clear(book)
o	int eb_is_bound(book)
o	string eb_path(book)
o	int eb_disc_type(book)
o	int eb_character_code(book)
filename.c:
-	int eb_catalog_filename(EB_Book *)
-	int eb_appendix_catalog_filename(EB_Appendix *)
-	int eb_canonicalize_filename(char *)
-	void eb_fix_filename(EB_Book *, char *)
-	void eb_fix_appendix_filename(EB_Appendix *, char *)
graphic.c:
o	int eb_have_graphic_search(book)
keyword.c:
o	int eb_have_keyword_search(book)
menu.c:
o	int eb_have_menu(book)
o	(int,int) eb_menu(book)
multi.c:
-	int eb_initialize_multi_search(EB_Book *)
o	int eb_have_multi_search(book)
-	int eb_multi_search_count(EB_Book *)
-	int eb_multi_search_list(EB_Book *, EB_Multi_Search_Code *)
-	int eb_multi_entry_count(EB_Book *, EB_Multi_Search_Code)
-	int eb_multi_entry_list(EB_Book *, EB_Multi_Search_Code,
	    EB_Multi_Entry_Code *)
-	const char *eb_multi_entry_label(EB_Book *, EB_Multi_Search_Code,
	    EB_Multi_Entry_Code)
-	int eb_multi_entry_have_exactword_search(EB_Book *,
	    EB_Multi_Search_Code, EB_Multi_Entry_Code)
-	int eb_multi_entry_have_word_search(EB_Book *, EB_Multi_Search_Code,
	    EB_Multi_Entry_Code)
-	int eb_multi_entry_have_endword_search(EB_Book *, EB_Multi_Search_Code,
	    EB_Multi_Entry_Code)
-	int eb_multi_entry_have_keyword_search(EB_Book *, EB_Multi_Search_Code,
	    EB_Multi_Entry_Code)
search.c:
o	int eb_have_exactword_search(book)
o	int eb_search_exactword(book, string)
o	int eb_have_word_search(book)
o	int eb_search_word(book, string)
o	int eb_have_endword_search(book)
o	int eb_search_endword(book, string)
o	((int,int)(int,int))[] eb_hit_list(book)
subbook.c:
o	int eb_initialize_all_subbooks(book)
o	int eb_subbook_count(book)
o	int[] eb_subbook_list(book)
o	int eb_subbook(book)
o	string eb_subbook_title(book)
o	string eb_subbook_title2(book, int)
o	string eb_subbook_directory(book)
o	string eb_subbook_directory2(book, int)
o	int eb_set_subbook(book, int)
o	void eb_unset_subbook(book)

[appendix.h]
appendix.c:
n	void eb_initialize_appendix(appendix)
o	void eb_clear_appendix(appendix)
o	void eb_suspend_appendix(appendix)
o	int eb_bind_appendix(appendix, const char *)
o	int eb_is_appendix_bound(appendix)
o	string eb_appendix_path(appendix)
appsub.c:
o	int eb_initialize_appendix_subbook(appendix)
o	int eb_initialize_all_appendix_subbooks(appendix)
o	int eb_appendix_subbook_count(appendix)
o	int[] eb_appendix_subbook_list(appendix)
o	int eb_appendix_subbook(appendix)
o	int eb_appendix_subbook2(appendix, string)
o	string eb_appendix_subbook_directory(appendix)
o	string eb_appendix_subbook_directory2(appendix, int)
o	int eb_set_appendix_subbook(appendix, int)
o	void eb_unset_appendix_subbook(appendix)
stopcode.c:
o	int eb_have_stopcode(EB_Appendix *)
o	int eb_stopcode(EB_Appendix *)
narwalt.c:
o	int eb_have_narrow_alt(EB_Appendix *)
o	int eb_narrow_alt_start(EB_Appendix *)
o	int eb_narrow_alt_end(EB_Appendix *)
o	int eb_narrow_alt_character_text(EB_Appendix *, int, char *)
o	int eb_forward_narrow_alt_character(EB_Appendix *, int, int)
o	int eb_backward_narrow_alt_character(EB_Appendix *, int, int)
widealt.c:
o	int eb_have_wide_alt(EB_Appendix *)
o	int eb_wide_alt_start(EB_Appendix *)
o	int eb_wide_alt_end(EB_Appendix *)
o	int eb_wide_alt_character_text(EB_Appendix *, int, char *)
o	int eb_forward_wide_alt_character(EB_Appendix *, int, int)
o	int eb_backward_wide_alt_character(EB_Appendix *, int, int)

[text.h]
hook.c:
o	int eb_hook_euc_to_ascii(book, appendix, buffer, int, int[])
o	int eb_hook_stopcode(book, appendix, buffer, int, int[])
o	int eb_hook_stopcode_dummy(book, appendix, buffer, int, int[])
o	int eb_hook_stopcode_mixed(book, appendix, buffer, int, int[])
o	int eb_hook_narrow_character_text(book, appendix, buffer, int, int[])
o	int eb_hook_wide_character_text(book, appendix, buffer, int, int[])
o	int eb_hook_empty(book, appendix, buffer, int, int[])
text.c:
n	void eb_initialize_hookset(hookset)
o	int eb_set_hook(hookset, (int, callable))
o	int eb_set_hooks(hookset, (int, callable)[])
o	int eb_seek(book, (int, int))
o	string eb_text(book, appendix, hookset)
o	string eb_heading(book, appendix, hookset)
o	string eb_rawtext(book, int)
uint.c:
-	unsigned eb_uint1(const char *)
-	unsigned eb_uint2(const char *)
-	unsigned eb_uint4(const char *)
-	unsigned eb_bcd2(const char *)
-	unsigned eb_bcd4(const char *)
-	unsigned eb_bcd6(const char *)

[error.h]
error.c:
o	string eb_error_message()
o	string eb_error_message2(int)

[language.h]
language.c:
-	int eb_language_count(EB_Book *)
-	int eb_language_list(EB_Book *, EB_Language_Code *)
-	int eb_have_language(EB_Book *, EB_Language_Code)
-	EB_Language_Code eb_language(EB_Book *)
-	const char *eb_language_name(EB_Book *)
-	const char *eb_language_name2(EB_Book *, EB_Language_Code)
-	int eb_set_language(EB_Book *, EB_Language_Code)
-	void eb_unset_language(EB_Book *)
message.c:
-	int eb_message_count(EB_Book *)
-	int eb_message_list(EB_Book *, EB_Message_Code *)
-	int eb_have_message(EB_Book *, EB_Message_Code)
-	const char *eb_message(EB_Book *, EB_Message_Code)

[font.h]
bitmap.c:
-	int eb_narrow_font_xbm_size(EB_Font_Code)
-	int eb_narrow_font_xpm_size(EB_Font_Code)
-	int eb_narrow_font_gif_size(EB_Font_Code)
-	int eb_wide_font_xbm_size(EB_Font_Code)
-	int eb_wide_font_xpm_size(EB_Font_Code)
-	int eb_wide_font_gif_size(EB_Font_Code)
-	size_t eb_bitmap_to_xbm(char *, const char *, int, int)
-	size_t eb_bitmap_to_xpm(char *, const char *, int, int)
-	size_t eb_bitmap_to_gif(char *, const char *, int, int)
font.c:
-	EB_Font_Code eb_font(EB_Book *)
-	int eb_set_font(EB_Book *, EB_Font_Code)
-	void eb_unset_font(EB_Book *)
-	int eb_font_count(EB_Book *)
-	int eb_font_list(EB_Book *, EB_Font_Code *)
-	int eb_have_font(EB_Book *, EB_Font_Code)
narwfont.c:
-	int eb_have_narrow_font(EB_Book *)
-	int eb_narrow_font_width(EB_Book *)
-	int eb_narrow_font_width2(EB_Font_Code)
-	int eb_narrow_font_size(EB_Book *)
-	int eb_narrow_font_size2(EB_Font_Code)
-	const char *eb_narrow_font_filename(EB_Book *)
-	const char *eb_narrow_font_filename2(EB_Book *, EB_Font_Code)
-	int eb_narrow_font_start(EB_Book *)
-	int eb_narrow_font_end(EB_Book *)
-	int eb_narrow_font_character_bitmap(EB_Book *, int, char *)
-	int eb_forward_narrow_font_character(EB_Book *, int, int)
-	int eb_backward_narrow_font_character(EB_Book *, int, int)
widefont.c:
-	int eb_have_wide_font(EB_Book *)
-	int eb_wide_font_width(EB_Book *)
-	int eb_wide_font_width2(EB_Font_Code)
-	int eb_wide_font_size(EB_Book *)
-	int eb_wide_font_size2(EB_Font_Code)
-	const char *eb_wide_font_filename(EB_Book *)
-	const char *eb_wide_font_filename2(EB_Book *, EB_Font_Code)
-	int eb_wide_font_start(EB_Book *)
-	int eb_wide_font_end(EB_Book *)
-	int eb_wide_font_character_bitmap(EB_Book *, int, char *)
-	int eb_forward_wide_font_character(EB_Book *, int, int)
-	int eb_backward_wide_font_character(EB_Book *, int, int)

*/
